@@ -21,14 +21,20 @@
 //
 //
 // Author: Juan Linietsky <coding@reduz.com.ar>, (C) 2003
+// Maintainer: J. Phelps <phelpsj@nuvox.net>
+// AKA Godless Infidel <godless@users.sf.net> 2007 (Copyright still held
+// by Linietsky)
 //
 // Copyright: See COPYING file that comes with this distribution
 //
 //
 
 #define APP_NAME_ "CheeseTracker "
-#define APP_VERSION_ "v0.9.9"
+#define APP_VERSION_ "0.9.15.2"
 #define APP_AUTHOR_ "(c) 1999,2004 Juan Linietsky"
+#define APP_RELEASE_DATE_ "Sun Aug 26 15:12:13 EDT 2007"
+#define APP_MAINTAINER_ "J Phelps <godless@users.sf.net>"
+#define APP_WEBSITE_ "http://cheesetracker.sf.net/"
 
 #include "mdi_main_window.h"
 #include "interface__QT/icons/control_play_prev.xpm"
@@ -46,6 +52,7 @@
 #include "interface__QT/icons/file_disk_close.xpm"
 
 #include "interface_binds/interface_help_text.h"
+#include "trackercore/Error.h"
 
 #include "icons/cheese_48x48.xpm"
 //#include "icons/cheese_16x16.xpm"
@@ -92,6 +99,8 @@ void MDI_Sub_Window::update_display_box() {
  	static char aux_msg_buffer[200];
 	int aux_playing_channels;
 
+	aux_msg_buffer[199]=0;
+
 	aux_playing_channels=interface->get_tracker()->mixer.get_active_voices_count();
 	int aux_playing_buffers=interface->get_tracker()->mixer.get_active_buffers_count();
 
@@ -101,24 +110,29 @@ void MDI_Sub_Window::update_display_box() {
 
 	 if (player->is_playing()) {
 
-		if (player->get_play_mode()==Player_Data::PLAY_SONG) {
+		if (player->get_current_pattern() != EMPTY_FIELD &&
+		    player->get_play_mode()==Player_Data::PLAY_SONG) {
 
-			sprintf(aux_msg_buffer,"Status: Playing, Order: %i/%i, Pattern %i, Row %i/%i, %i Voices, %i Buffers.",player->get_current_order(),song->get_used_orders_amount(),player->get_current_pattern(),player->get_current_row(),song->get_pattern(player->get_current_pattern())->get_length(),aux_playing_channels,aux_playing_buffers);
+			snprintf(aux_msg_buffer, sizeof(aux_msg_buffer)-1,"Status: Playing, Order: %i/%i, Pattern %i, Row %i/%i, %i Voices, %i Buffers.",player->get_current_order(),song->get_used_orders_amount(),player->get_current_pattern(),player->get_current_row(),song->get_pattern(player->get_current_pattern())->get_length(),aux_playing_channels,aux_playing_buffers);
 		} else {
 
-			sprintf(aux_msg_buffer,"Status: Playing, Pattern %i, Row %i/%i, %i Voices, %i Buffers.",player->get_current_pattern(),player->get_current_row(),song->get_pattern(player->get_current_pattern())->get_length(),aux_playing_channels,aux_playing_buffers);
+			if(player->get_current_pattern() != EMPTY_FIELD)
+				snprintf(aux_msg_buffer, sizeof(aux_msg_buffer)-1, "Status: Playing, Pattern %i, Row %i/%i, %i Voices, %i Buffers.",player->get_current_pattern(),player->get_current_row(),song->get_pattern(player->get_current_pattern())->get_length(),aux_playing_channels,aux_playing_buffers);
+			else
+				snprintf(aux_msg_buffer, sizeof(aux_msg_buffer)-1,
+				"Status: Playing an empty pattern\n");
 		}
 
 	} else if (aux_playing_channels>0) {
 
-		sprintf(aux_msg_buffer,"Status: Active, %i Voices, %i Buffers.",aux_playing_channels,aux_playing_buffers);
+		snprintf(aux_msg_buffer, sizeof(aux_msg_buffer)-1, "Status: Active, %i Voices, %i Buffers.",aux_playing_channels,aux_playing_buffers);
 
 	} else {
 
 		if (aux_playing_buffers)
-			sprintf(aux_msg_buffer,"Status: Idle, %i Buffers.",aux_playing_buffers);
+			snprintf(aux_msg_buffer, sizeof(aux_msg_buffer)-1, "Status: Idle, %i Buffers.",aux_playing_buffers);
 		else
-			sprintf(aux_msg_buffer,"Status: Idle.");
+			snprintf(aux_msg_buffer, sizeof(aux_msg_buffer)-1, "Status: Idle.");
 	}
 
 	if (message_cache!=aux_msg_buffer) {
@@ -144,10 +158,16 @@ MDI_Sub_Window::MDI_Sub_Window( QWidget* parent, const char* name, int wflags )
 
 void MDI_Sub_Window::open_song(string p_name) {
 
-	Loader::Error  res=interface->open_song(p_name);
-	if (res) {
+	try {
+		Loader::Error  res=interface->open_song(p_name);
+		if (res) {
 
-		QMessageBox::critical( this, "Error Loading Song",Loader::error_string[res],"Ok :(");
+			QMessageBox::critical( this, "Error Loading Song",Loader::error_string[res],"Ok :(");
+			close();
+		}
+	}
+	catch (Error E) {
+		QMessageBox::critical(this, "Error Loading Song", E.what(), "Ok :(");
 		close();
 	}
 
@@ -243,7 +263,10 @@ void MDI_Main_Window::timer_callback() {
 
 }
 
-
+void MDI_Main_Window::ctrl_i_make_instrument()
+{
+	sample_edit_menu_item_activated(SampleEdit__MakeInstrument);
+}
 
 void MDI_Main_Window::pattern_edit_menu_item_activated(int p_id) {
 
@@ -619,6 +642,26 @@ MDI_Main_Window::MDI_Main_Window() : QMainWindow( 0, "Main Window"/*, WDestructi
 	file_open_action->addTo(file_menu_toolbar);
 	file_open_action->addTo(file);
 
+	// F9_file_open_action is a hidden action: It has a
+	// key binding, but no corresponding menu entry.
+
+	QAction * F9_file_open_action =
+		new QAction("Open Song",
+		            QPixmap((const char**)file_disk_open_xpm), "",
+		            Key_F9, this);
+
+	QObject::connect(F9_file_open_action, SIGNAL( activated() ), this,
+	                 SLOT(load()) );
+
+	// Hidden actions must go into the hidden action list to prevent
+	// a memory leak.
+
+	hidden_actions.push_back(F9_file_open_action);
+
+
+	// Make Ctrl+I load an instrument. This requires a QAction with
+	// no associated menu item.
+
 	QAction * file_save_action = new QAction("Save",QPixmap((const char**)file_disk_xpm),"&Save",CTRL+Key_S,this);
 	QObject::connect( file_save_action, SIGNAL( activated() ) , this, SLOT(save()) );
 	file_save_action->addTo(file_menu_toolbar);
@@ -651,6 +694,8 @@ MDI_Main_Window::MDI_Main_Window() : QMainWindow( 0, "Main Window"/*, WDestructi
 	pattern_edit_menu->setAccel(Qt::ALT+Qt::Key_B ,Editor::Pattern_MARK_BLOCK_BEGIN);
 	pattern_edit_menu->insertItem("Mark- Block End",Editor::Pattern_MARK_BLOCK_END);
 	pattern_edit_menu->setAccel(Qt::ALT+Qt::Key_E ,Editor::Pattern_MARK_BLOCK_END);
+	pattern_edit_menu->insertItem("Mark- Sixteen notes", Editor::Pattern_MARK_COLUMN_SIXTEEN);
+	pattern_edit_menu->setAccel(Qt::ALT+Qt::Key_D, Editor::Pattern_MARK_COLUMN_SIXTEEN);
 	pattern_edit_menu->insertItem("Mark- Toggle Column/All",Editor::Pattern_MARK_COLUMN_ALL);
 	pattern_edit_menu->setAccel(Qt::ALT+Qt::Key_L ,Editor::Pattern_MARK_COLUMN_ALL);
 	pattern_edit_menu->insertSeparator();
@@ -718,11 +763,13 @@ MDI_Main_Window::MDI_Main_Window() : QMainWindow( 0, "Main Window"/*, WDestructi
 	sample_edit_menu->insertItem("Sample- Save",SampleEdit__Save);
 	sample_edit_menu->insertSeparator();
 	sample_edit_menu->insertItem("Sample- Make Instrument",SampleEdit__MakeInstrument);
+	sample_edit_menu->setAccel(CTRL+Key_I, SampleEdit__MakeInstrument);
 	sample_edit_menu->insertSeparator();
 	sample_edit_menu->insertItem("Sample- Clear Unused Samples",SampleEdit__ClearUnused);
 	sample_edit_menu->insertItem("Sample- Clear Default Pannings",SampleEdit__ClearDefaultPanning);
 	QObject::connect(sample_edit_menu,SIGNAL(activated(int)),this,SLOT(sample_edit_menu_item_activated(int)));
 
+
         /* Instrument Edit */
 
 	instrument_edit_menu = new QPopupMenu( this );
@@ -884,7 +931,10 @@ void MDI_Main_Window::help_menu_item_activated(int p_id) {
 
 			string about_msg=(string)APP_NAME_ + (string)APP_VERSION_ +(string)"\n";
 			about_msg+=(string)APP_AUTHOR_+(string)"\n";
-			about_msg+="http://reduz.com.ar/cheesetracker";
+			about_msg+="Released on ";
+			about_msg+=(string)APP_RELEASE_DATE_+(string)"\n";
+			about_msg+=(string)"by " + (string)APP_MAINTAINER_ + "\n";
+			about_msg+=(string)APP_WEBSITE_;
 			QMessageBox::information( this, "About",about_msg.c_str(),"Ok");
 		} break;
 	}
@@ -982,5 +1032,10 @@ void MDI_Main_Window::about()
 }
 
 MDI_Main_Window::~MDI_Main_Window() {
+	list<QAction*>::iterator ix;
+
+	for(ix=hidden_actions.begin(); ix != hidden_actions.end(); ix++) {
+		delete (*ix);
+	}
 
 }
