@@ -33,7 +33,10 @@
  *                                                                         *
  ***************************************************************************/
 
+
+#include <algorithm>
 #include "saver_it.h"
+#include "ns_autoptr.h"
 
 
 #define BITBOOL(m_exp) ((m_exp)?1:0)
@@ -178,7 +181,7 @@ void Saver_IT::write_sample_internal(int p_sample_index,bool p_write_data) {
 	//FLAGS!
 	tmpint=BITBOOL(song->get_sample(p_sample_index)->in_use) |
      	       (BITBOOL(song->get_sample(p_sample_index)->data.is_16bit())<<1)|
-     	      // (song->get_sample(p_sample_index)->data.is_stereo<<2)|
+               BITBOOL(song->get_sample(p_sample_index)->data.num_channels() > 1) << 2  |
 		// Flag 3 is zero - sample uncompressed
      	       (BITBOOL(song->get_sample(p_sample_index)->data.is_loop_enabled())<<4)|
 //     	       (BITBOOL(song->get_sample(p_sample_index)->data.sustain_loop_on)<<5)|
@@ -194,10 +197,20 @@ void Saver_IT::write_sample_internal(int p_sample_index,bool p_write_data) {
 	Uint8 defpan=song->get_sample(p_sample_index)->def_panning;
 	defpan|=song->get_sample(p_sample_index)->def_panning_on?128:0;
 	writer.store_byte(defpan);
+
+	Mutex_Lock_Container *sample_data_lock;
+
+	// Critical section!
+
+	sample_data_lock = song->get_sample(p_sample_index)->data.lock(__FILE__, __LINE__);
 	writer.store_dword(song->get_sample(p_sample_index)->data.get_size());
 	writer.store_dword(song->get_sample(p_sample_index)->data.get_loop_begin());
 	writer.store_dword(song->get_sample(p_sample_index)->data.get_loop_end());
 	writer.store_dword(song->get_sample(p_sample_index)->data.get_c5_freq());
+	delete sample_data_lock;
+
+	// End critical section!
+
 	writer.store_dword(0);
 	writer.store_dword(0);
 //	writer.store_dword(song->get_sample(p_sample_index)->data.sustain_loop_begin);
@@ -210,11 +223,34 @@ void Saver_IT::write_sample_internal(int p_sample_index,bool p_write_data) {
 	writer.store_byte(song->get_sample(p_sample_index)->vibrato_type);
 
 	/** SAMPLE IS STORED UNCOMPRESSED **/
+	// CHANNELS AFTER THE SECOND ONE ARE DISCARDED
+
  	if (p_write_data) {
 
 		if (song->get_sample(p_sample_index)->in_use) {
 
-			writer.store_byte_array((Uint8*)song->get_sample(p_sample_index)->data.get_data_16(),song->get_sample(p_sample_index)->data.is_16bit()?song->get_sample(p_sample_index)->data.get_size()*2:song->get_sample(p_sample_index)->data.get_size());
+			Sample *smp = song->get_sample(p_sample_index);
+			// Touch the sample so that we don't access it
+			// while it's being modified or while it's in
+			// use_fixedpoint() mode.
+			Mutex_Lock_Container *read_lock = smp->data.lock(__FILE__, __LINE__);
+			// Ensure that the touch will be released at the end
+			// of this if-block.
+			ns_autoptr<Mutex_Lock_Container> ns_read_lock;
+			ns_read_lock.ptr_new(read_lock);
+
+			for(size_t ix=0; ix< smp->data.get_size(); ix++) {
+				const sample_int_t *buffer = smp->data.get_int_sample();
+
+				for(size_t chan=0;
+				    chan < std::min<size_t>(2, smp->data.num_channels());
+				    chan++) {
+					if(smp->data.is_16bit())
+						writer.store_word(CONVERT_TO_TYPE(Sint16, buffer[chan]));
+					else
+						writer.store_byte(CONVERT_TO_TYPE(Sint8, buffer[chan]));
+				}
+			}
 		}
   	}
 
@@ -444,7 +480,7 @@ void Saver_IT::write_pattern(int p_pattern) {
 				int flags=0;
 				Note src_note=song->get_pattern(p_pattern)->get_note(j,i);
 
-				//Check for what changed, and fill cache!
+				// Check for what changed, and fill cache!
 
 				if (src_note.note!=EMPTY_FIELD) {
 
@@ -713,8 +749,8 @@ int Saver_IT::save_instrument(const char *p_filename,int p_instrument_index) {
         //replace sample stuff for 1->x stuff
 	for (i=0;i<Note::NOTES;i++) {
 
-   		int sampnumb=instr.get_layer(0)->sample_number[i];
-     		int newnumb;
+   		Uint8 sampnumb=instr.get_layer(0)->sample_number[i];
+     		Uint8 newnumb=EMPTY_FIELD;
                 bool has_it=false;
 
                 //if sample slot is empty or points to an ununsed sample, skip it
@@ -753,8 +789,6 @@ int Saver_IT::save_instrument(const char *p_filename,int p_instrument_index) {
 
 	for (i=0;i<(int)samples_used.size();i++) {
 
-
-
 		write_sample_internal(samples_used[i],false);
   		current_offset=writer.get_file_pos()-8;
 	 	sample_data_ptr_list.push_back(current_offset);
@@ -765,8 +799,23 @@ int Saver_IT::save_instrument(const char *p_filename,int p_instrument_index) {
    		int prev_offset,aux_offset;
 
      		prev_offset=writer.get_file_pos();
-		
-		writer.store_byte_array((Uint8*)song->get_sample(samples_used[i])->data.get_data_16(),song->get_sample(samples_used[i])->data.is_16bit()?song->get_sample(samples_used[i])->data.get_size()*2:song->get_sample(samples_used[i])->data.get_size());
+
+		Sample *smp = song->get_sample(samples_used[i]);
+		Mutex_Lock_Container *lock = smp->data.lock(__FILE__, __LINE__);
+		ns_autoptr<Mutex_Lock_Container> ns_lock;
+		ns_lock.ptr_new(lock);
+
+		for(size_t ix=0; ix< smp->data.get_size(); ix++) {
+			const sample_int_t *buffer = smp->data.get_int_sample();
+			for(size_t chan=0;
+			    chan< std::min<size_t>(2, smp->data.num_channels());
+			    chan++) {
+				if(smp->data.is_16bit())
+					writer.store_word(CONVERT_TO_TYPE(Sint16, buffer[chan]));
+				else
+					writer.store_byte(CONVERT_TO_TYPE(Sint8, buffer[chan]));
+			}
+		}
 
   		aux_offset=writer.get_file_pos();
 
