@@ -24,7 +24,8 @@ class Effect_Source_Internal : public Effect_Source {
 
 	struct Constructor {
 
-		Effect*(*constructor_ptr)();
+		Effect*(*constructor_ptr)(Effect::Parameters*);
+		Effect::Parameters*(*params_constructor_ptr)();
 		string name;
 		string var_name;
 	};
@@ -32,12 +33,13 @@ class Effect_Source_Internal : public Effect_Source {
 
 public:
 	int get_effect_count();
-	string get_effect_name(int p_which);
+	string get_effect_identifier_name(int p_which);
 	string get_effect_display_name(int p_which);
-	Effect *instantiate_effect(int p_which);
+	Effect *instantiate_effect(int p_which,Effect::Parameters *p_params);
+	Effect::Parameters *instantiate_effect_parameters(int p_which);
 	string get_name();
 
-	void register_effect_constructor( string p_name,string p_var_name,Effect*(*p_effect_constructor)() );
+	void register_effect_constructor( string p_name,string p_var_name,Effect*(*p_effect_constructor)(Effect::Parameters*),Effect::Parameters*(*p_effect_params_constructor)() );
 	Effect_Source_Internal();
 	~Effect_Source_Internal();
 };
