@@ -21,16 +21,6 @@
 	using std::string;
 #include "hash_stuff.h"
 
-#if (__GNUC__ == 3) || (__GNUC__ > 3)
-
-	#include <ext/hash_map>
-	using __gnu_cxx::hash;
-#else
-
-	#include <hash_map>
-	using std::hash;
-#endif
-
 
 struct String_Hash {
 
