@@ -131,7 +131,7 @@ bool Envelope_Point_Editor::event ( QEvent * e ) {
 			if ( ((int)gx!=envelope->get_node_offset(grab.node)) || ((int)gy!=envelope->get_node_height(grab.node)) ) {
 
 				//avoid pointless overdraw
-				if (envelope_lock) envelope_lock->grab();
+				if (envelope_lock) envelope_lock->grab(__FILE__, __LINE__);
 				envelope->set_node_offset(grab.node,(int)gx,(int)gy);
 				if (envelope_lock) envelope_lock->release();
 				if (adjust_zoom_to_window())
@@ -208,7 +208,7 @@ bool Envelope_Point_Editor::event ( QEvent * e ) {
 					gy-=env_normalize;
 
 
-					if (envelope_lock) envelope_lock->grab();
+					if (envelope_lock) envelope_lock->grab(__FILE__, __LINE__);
                                 	int which=envelope->add_node_at_offset((int)gx,(int)gy);
 					if (envelope_lock) envelope_lock->release();
 
@@ -231,7 +231,7 @@ bool Envelope_Point_Editor::event ( QEvent * e ) {
 
         			if (closest_node!=-1) {
 
-					if (envelope_lock) envelope_lock->grab();
+					if (envelope_lock) envelope_lock->grab(__FILE__, __LINE__);
         				envelope->delete_node(closest_node);
 					if (envelope_lock) envelope_lock->release();
 					update();
@@ -321,6 +321,7 @@ void Envelope_Point_Editor::paint_envelope_points(QPainter &painter) {
 	const QFontMetrics &display_font_metrics=painter.fontMetrics();
 
 	char buf[200];
+	buf[199]=0;
 
 //	painter.setPen(QColor(100, 200, 100));
 
@@ -345,7 +346,7 @@ void Envelope_Point_Editor::paint_envelope_points(QPainter &painter) {
 
 		int real_screen_offset=int((float)i/zoom);
 
-		sprintf(buf,"%i",(int)i);
+		snprintf(buf, sizeof(buf)-1,"%i",(int)i);
 		if (i>0.0f)
 			painter.drawText(real_screen_offset-display_font_metrics.width(buf)/2,cur_height+display_font_metrics.height(),buf);
 	}
@@ -354,10 +355,10 @@ void Envelope_Point_Editor::paint_envelope_points(QPainter &painter) {
 
 	if (grab.moving_point) {
 
-		sprintf(buf,"Zoom %i%%, %i nodes. Moving node %i, pos: %i,%i",int(zoom*100.0),envelope->get_node_count(),grab.node,envelope->get_node_offset(grab.node),envelope->get_node_height(grab.node));
+		snprintf(buf, sizeof(buf)-1,"Zoom %i%%, %i nodes. Moving node %i, pos: %i,%i",int(zoom*100.0),envelope->get_node_count(),grab.node,envelope->get_node_offset(grab.node),envelope->get_node_height(grab.node));
 	} else {
 
-		sprintf(buf,"Zoom %i%%, %i nodes. ",int(zoom*100.0),envelope->get_node_count());
+		snprintf(buf, sizeof(buf)-1,"Zoom %i%%, %i nodes. ",int(zoom*100.0),envelope->get_node_count());
 	}
 
 
