@@ -45,13 +45,16 @@ friend class Sorter;
 public:
 
 	int get_effect_count();
-	string get_effect_name(int p_which);
+	string get_effect_identifier_name(int p_which);
 	string get_effect_display_name(int p_which);
-	Effect *instantiate_effect(int p_which);
+
+	virtual Effect *instantiate_effect(int p_which,Effect::Parameters *p_params); //null params means use internal
+	virtual Effect::Parameters *instantiate_effect_parameters(int p_which);
+	
 	string get_name();
 
 	LADSPA_Effect_Source();
-	~LADSPA_Effect_Source();
+	virtual ~LADSPA_Effect_Source();
 
 };
 
